/**
 * Post-install script to pre-configure the config files.
 */
function postInstall()
{	
	
	log( 2, "Starting postInstall()..." );
	
	
	try
	{
		// --- Pre-configure the server.xml file ---
		var serverXmlFilename ;
		var licensePropertiesFileName;
		if (ncipackage.shortArch == "win32" ) {
			serverXmlFilename = getNchome() + "\\guifoundation\\conf\\server.xml";
			licensePropertiesFileName = getNchome() + "\\guifoundation\\conf\\license.properties";
		} else {
			serverXmlFilename = getNchome() + "/guifoundation/conf/server.xml";
			licensePropertiesFileName = getNchome() + "/guifoundation/conf/license.properties";
		}
		
		
		// get contents of file
		log( 2, "Pre-configuring " + serverXmlFilename );
		var serverXmlContents = readFromFile( serverXmlFilename );
		backupFile( serverXmlFilename );
		var newServerXmlContents = processServerXml( serverXmlContents );
		writeToFile( serverXmlFilename, newServerXmlContents );		
		log( 3, serverXmlFilename + " pre-configuration completed." );
		

		// --- Pre-configure the license.properties file ---
		var licensePropertiesContents = readFromFile( licensePropertiesFileName );
		backupFile( licensePropertiesFileName );
		var newLicensePropertiesContents = processLicenseProperties( licensePropertiesContents );
		writeToFile( licensePropertiesFileName, newLicensePropertiesContents );
		
		
		// Do no bother for the asa file on windows...
		if ( ncipackage.shortArch != "win32" ) {
			// --- Pre-configure the asa.sh file ---
			var asaScriptFileName = getNchome() + "/guifoundation/bin/asa.sh";
			log(2, "Pre-configuring " + asaScriptFileName );

			// get contents of file
			var asaScriptContents = readFromFile( asaScriptFileName );

			// backup file
			backupFile( asaScriptFileName );

			// parse contents of file and replace strings
			var newAsaScriptContents = processAsaScript( asaScriptContents );

			// write it back to the file
			writeToFile( asaScriptFileName, newAsaScriptContents );
			log( 3, asaScriptFileName + " pre-configuration completed." );		
		}

		// Only bother with the Torque.properties on HPUX OR AIX
		if ( ncipackage.shortArch == "hpux11" || ncipackage.shortArch == "aix5") {
			// --- Pre-configure the file ---
			var torquePropsFileName = getNchome() + "/guifoundation/webapps/desktop/WEB-INF/conf/Torque.properties";
			log(2, "Pre-configuring " + torquePropsFileName );

			// get contents of file
			var torquePropsContents = readFromFile( torquePropsFileName );
			
			// backup the file
			backupFile( torquePropsFileName );
			
			var newTorquePropsContents = processTorqueProps( torquePropsContents );
			writeToFile( torquePropsFileName, newTorquePropsContents);
			log(3, torquePropsFileName + "preconfiguration completed");
		}
	}	
	catch ( e )
	{
		log( 0, e );
	}
	
	log( 3, "postInstall() completed." );
}

/**
 * Read the contents of a file.
 *
 * @param filename (String) The full path of the file to read from.
 * @return (String) Contents of the file.
 */
function readFromFile( filename )
{
	// Read the file.
	log( 2, "Reading file: " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	// NOTE: This will not work with Mac!
	file.initWithPath( filename );

	// Use IOService to convert the file to a file URI.
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService( Components.interfaces.nsIIOService );
	var fileUri = ioService.newFileURI( file.QueryInterface( Components.interfaces.nsIFile ) );

	// Get a channel from the URI.
	var channel = ioService.newChannelFromURI( fileUri );

	// Create an input stream.
	var inStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance( Components.interfaces.nsIScriptableInputStream );

	var fileContents = "";

	try
	{
		inStream.init( channel.open() );

		var bytes;

		while ( (bytes = inStream.available()) > 0 )
		{
			fileContents += inStream.read( bytes );
		}
	}
	catch ( e )
	{
		log( 0, e );
	}
	finally
	{
		inStream.close();
	}
	
	log( 3, "Read completed." );
	
	return fileContents;
}

/**
 * Backup a file by appending ".bak" to the filename.
 *
 * @param filename (String) The full path of the file to backup.
 */
function backupFile( filename )
{
	log( 2, "Backing up " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	file.initWithPath( filename );
	
	file.moveTo( file.parent, file.leafName + ".bak" );
	
	log( 3, "Backup completed." );
}

/**
 * Process the content of the server.xml file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processServerXml( fileContents )
{
	log( 2, "Processing definitions file..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

	// Replace port name (in all places).
	var regex = /8080/g;
	var newContentsTmp = fileContents.replace( regex,propMgr.GetProp( "GUIFOUNDATION.WEBSERVER.PORT" ).value );

	// replace shutdown port
	var regex = /8005/g;
	var newContents = newContentsTmp.replace( regex,propMgr.GetProp( "GUIFOUNDATION.WEBSERVER.SHUTDOWNPORT" ).value );
	

	return newContents;
}

/**
 * Process the content of the asa.sh file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processAsaScript( fileContents )
{
	log( 2, "Processing definitions file..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );
	
	
	// generate a Hash for NCHOME
	// var uniqueString = sha1Hash( propMgr.GetProp( "[$NCHOME]" ).value );
	
	//var dns = Components.classes["@mozilla.org/network/dns-service;1"].createInstance(Components.interfaces.nsIDNSService);
	//var myhostname = dns.myHostName ;
	//dns.shutdown();
	
	var uniqueString = propMgr.GetProp( "GUIFOUNDATION.WEBSERVER.PORT" ).value ;
	
	var finalDbName = "NGF-DB-" + "-" + uniqueString ;

	// Replace database name (in all places).
	var regex = /ServerName=MachDesktop/g;
	var newContentsTmp = fileContents.replace( regex, "ServerName=" + finalDbName );

	// Replace database port.
	regex = /2638/g;
	newContents = newContentsTmp.replace( regex,propMgr.GetProp( "GUIFOUNDATION.DATABASE.PORT" ).value );

	return newContents;
}

/**
 * Process the content of the Torque.properties file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processTorqueProps( fileContents )
{
        log( 2, "Processing definitions file..." );

        var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );
	

	var regex = /torque.dsfactory.default.connection.url=jdbc:odbc:DSN=MachDesktopDSN/g

	var newContents = fileContents.replace( regex, "torque.dsfactory.default.connection.url=jdbc:odbc:DSN=MachDesktopDSN;LINKS=tcpip(IP=localhost:" + propMgr.GetProp( "GUIFOUNDATION.DATABASE.PORT" ).value + ")" )

        return newContents;
}

/**
 * Process the content of the license.properties file.
 *
 * @param fileContents (String) The content to process.
 * @return (String) The processed content.
 */
function processLicenseProperties( fileContents )
{
	log( 2, "Processing definitions file..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );
	
	var licenseserver = propMgr.GetProp( "LICENSESERVER.PORT" ).value + "@" + propMgr.GetProp( "LICENSESERVER.HOST" ).value ;

	// Replace database (in all places).
	var regex = /(license.server:).*/g;
	var newContents = fileContents.replace( regex, "$1" + licenseserver );

	return newContents;
}


/**
 * Write content to a file. The file will have 664 permissions.
 *
 * @param filename (String) The full path of the file to read from.
 * @param content (String) The content to write.
 */
function writeToFile( filename, content )
{
	log( 2, "Writing to file: " + filename );

	// Create and initialise the file.
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
	file.initWithPath( filename );

	// To write to the file, make an output stream, initialize the stream with
	// the file, and write to it.
	var outStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
	
	try
	{
		outStream.init( file.QueryInterface( Components.interfaces.nsIFile ), 0x04 | 0x08, 0664, 0 );
		// octal 0664 gives "664" permissions.
		var result = outStream.write( content, content.length );
	}
	catch ( e )
	{
		// Log!
		log( 0, e );
	}
	finally
	{
		outStream.close();
	}
	
	log( 3, "Write completed." );
}


// Get $NCHOME.
function getNchome()
{
	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

	return propMgr.GetProp( "NCHOME.INSTALLDIR" ).value;
}


/**
 * Log a message.
 *
 * @param level (int) The level to log with. Valid levels are: 0 - Error,
 *        1 - Warning 2 - Info, 3 - Debug.
 * @param msg (String) The message to log.
 */
function log( level, msg )
{
	try
	{
		var logger = Components.classes["@micromuse.com/NCILog;1"].getService( Components.interfaces.nsINCILog);
		
		switch ( level )
		{
			case 0: logger.error( msg ); break;
			case 1: logger.warning( msg ); break;
			case 2: logger.info( msg ); break;
			case 3: logger.debug( msg ); break;
			default: break;
		}
	}
	catch ( e )
	{
		// ??
	}
}


/** SHA-1 hashing from http://www.movable-type.co.uk/scripts/SHA-1.html
*/

function sha1Hash(msg)
{
    // constants [4.2.1]
    var K = new Array(0x5a827999, 0x6ed9eba1, 0x8f1bbcdc, 0xca62c1d6);

    // PREPROCESSING 
 
    msg += String.fromCharCode(0x80); // add trailing '1' bit to string [5.1.1]

    // convert string msg into 512-bit/16-integer blocks arrays of ints [5.2.1]
    var l = Math.ceil(msg.length/4) + 2;  // long enough to contain msg plus 2-word length
    var N = Math.ceil(l/16);              // in N 16-int blocks
    var M = new Array(N);
    for (var i=0; i<N; i++) {
        M[i] = new Array(16);
        for (var j=0; j<16; j++) {  // encode 4 chars per integer, big-endian encoding
            M[i][j] = (msg.charCodeAt(i*64+j*4)<<24) | (msg.charCodeAt(i*64+j*4+1)<<16) | 
                      (msg.charCodeAt(i*64+j*4+2)<<8) | (msg.charCodeAt(i*64+j*4+3));
        }
    }
    // add length (in bits) into final pair of 32-bit integers (big-endian) [5.1.1]
    M[N-1][14] = ((msg.length-1) >>> 30) * 8;
    M[N-1][15] = ((msg.length-1)*8) & 0xffffffff;

    // set initial hash value [5.3.1]
    var H0 = 0x67452301;
    var H1 = 0xefcdab89;
    var H2 = 0x98badcfe;
    var H3 = 0x10325476;
    var H4 = 0xc3d2e1f0;

    // HASH COMPUTATION [6.1.2]

    var W = new Array(80); var a, b, c, d, e;
    for (var i=0; i<N; i++) {

        // 1 - prepare message schedule 'W'
        for (var t=0;  t<16; t++) W[t] = M[i][t];
        for (var t=16; t<80; t++) W[t] = ROTL(W[t-3] ^ W[t-8] ^ W[t-14] ^ W[t-16], 1);

        // 2 - initialise five working variables a, b, c, d, e with previous hash value
        a = H0; b = H1; c = H2; d = H3; e = H4;

        // 3 - main loop
        for (var t=0; t<80; t++) {
            var s = Math.floor(t/20); // seq for blocks of 'f' functions and 'K' constants
            T = (ROTL(a,5) + f(s,b,c,d) + e + K[s] + W[t]) & 0xffffffff;
            e = d;
            d = c;
            c = ROTL(b, 30);
            b = a;
            a = T;
        }

        // 4 - compute the new intermediate hash value
        H0 = (H0+a) & 0xffffffff;  // note 'addition modulo 2^32'
        H1 = (H1+b) & 0xffffffff; 
        H2 = (H2+c) & 0xffffffff; 
        H3 = (H3+d) & 0xffffffff; 
        H4 = (H4+e) & 0xffffffff;
    }

    return H0.toHexStr() + H1.toHexStr() + H2.toHexStr() + H3.toHexStr() + H4.toHexStr();
}

//
// function 'f' [4.1.1]
//
function f(s, x, y, z) 
{
    switch (s) {
    case 0: return (x & y) ^ (~x & z);
    case 1: return x ^ y ^ z;
    case 2: return (x & y) ^ (x & z) ^ (y & z);
    case 3: return x ^ y ^ z;
    }
}

//
// rotate left (circular left shift) value x by n positions [3.2.5]
//
function ROTL(x, n)
{
    return (x<<n) | (x>>>(32-n));
}

//
// extend Number class with a tailored hex-string method 
//   (note toString(16) is implementation-dependant, and 
//   in IE returns signed numbers when used on full words)
//
Number.prototype.toHexStr = function()
{
    var s="", v;
    for (var i=7; i>=0; i--) { v = (this>>>(i*4)) & 0xf; s += v.toString(16); }
    return s;
}
