/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2005, Micromuse, Ltd.                     */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* PropertyValue()
*
* Return the value of a property as a string.
*
* @param propName Name of the property to return the value of.
*
* @return The property value, or an empty string if the property was not found.
*
********************************************************************************
*/
function PropertyValue(propName)
{
    var answer = "";

    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    var prop = propMgr.GetProp(propName);
    if (prop)
    {
        answer = prop.value;
    }

    return answer;
}

/**
********************************************************************************
*
* ConfigureModelToMySQL()
*
* Function to run "ncp_model_to_mysql -batchconfigure" which sets up the MySQL
* database for Precision to use.
*
********************************************************************************
*/
function ConfigureModelToMySQL()
{
    //
    // Set up platform specific variables
    //
    var separator = "/";
    var topLine = "#!/bin/sh";
    var precisionHomeEnv = "PRECISION_HOME=" + GetNcHome() + "/precision\n"
                         + "export PRECISION_HOME";
    var ncpModelToMysqlPath = GetNcHome() + "/precision/bin/ncp_model_to_mysql";

    if (ncipackage.shortArch == "win32")
    {
        separator = "\\";
        topLine = "@echo off";
        precisionHomeEnv = "set PRECISION_HOME=" + GetNcHome() + "\\precision\n"
                         + "set NCHOME=" + GetNcHome() + "\n"
                         + "path " + GetNcHome() + "\\platform\\win32\\bin;%Path%"
        ncpModelToMysqlPath = GetNcHome()
                            + "\\precision\\platform\\win32\\bin\\ncp_model_to_mysql.exe";
    }

    //
    // We'll need the local host name.
    //
    var DNSService = Components.classes["@mozilla.org/network/dns-service;1"].
        getService(Components.interfaces.nsIDNSService);

    var hostName = DNSService.myHostName;

    try
    {
        //
        // First stage is to write a file containing the input for
        // ncp_model_to_mysql -batchconfigure.  This consists of 11 properties,
        // one per line, as follows:
        //
        //  1. Domain name
        //  2. Object server name
        //  3. Webtop Datasource name
        //  4. MySQL host name
        //  5. MySQL port
        //  6. Current MySQL root username
        //  7. Current MySQL root password
        //  8. MySQL user name to set up for normal operation
        //  9. MySQL password to set up for normal operation
        // 10. RVA host name
        // 11. RVA port
        //
        // It is assumed that RVA will run on the current box on port 7600.  The
        // other values come from installer properties.
        //
        var inputFileName = GetNcHome() + separator + "ncp_model_to_mysql.txt";

        var inputFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        inputFile.initWithPath(inputFileName);

        var contents = PropertyValue("PRECISION.DOMAIN0") + "\n"
                     + PropertyValue("OBJECTSERVER0.NAME") + "\n"
                     + PropertyValue("NCWDATASOURCE0.NAME") + "\n"
                     + PropertyValue("MYSQL41.HOST") + "\n"
                     + PropertyValue("MYSQL41.PORT") + "\n"
                     + PropertyValue("MYSQL41.ADMINUSERNAME") + "\n"
                     + PropertyValue("MYSQL41.ADMINPASSWORD") + "\n"
                     + PropertyValue("PRECISION.MYSQLUSERNAME") + "\n"
                     + PropertyValue("PRECISION.MYSQLPASSWORD") + "\n"
                     + hostName + "\n"
                     + "7600\n";

        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream.init(inputFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
        foStream.write(contents, contents.length);
        foStream.close();

        //
        // Now create a script to run "ncp_model_to_mysql -batchconfigure" after
        //
        var scriptFileName = GetNcHome() + separator + "batch_config.bat";

        var scriptFile = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        scriptFile.initWithPath(scriptFileName);

        var scriptContents = topLine + "\n"
                           + precisionHomeEnv + "\n"
                           + "\"" + ncpModelToMysqlPath + "\" -domain "
                           + PropertyValue("PRECISION.DOMAIN0")
                           + " -batchconfigure \"" + inputFileName + "\" > \""
                           + GetNcHome() + separator + "log" + separator
                           + "precision" + separator + "ncp_model_to_mysql."
                           + PropertyValue("PRECISION.DOMAIN0") + ".log\" 2>&1\n";

        var foStream2 = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream2.init(scriptFile, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
        foStream2.write(scriptContents, scriptContents.length);
        foStream2.close();

        //
        // Now run the script.
        //
        var process = Components.classes["@mozilla.org/process/util;1"].
            createInstance(Components.interfaces.nsIProcess);
        process.init(scriptFile);

        var args = [];
        process.run(true, args, args.length);

        //
        // Finally, clean up the temporary files.
        //
        inputFile.remove(false);
        scriptFile.remove(false);
    }
    catch(e)
    {
        Log(0, e);
    }
}

/**
********************************************************************************
*
* CreateCtrlServices()
*
* Function to create a CtrlServices.DOMAIN.cfg file, based on the default
* version but with "NCOMS" substituted for the name of the ObjectServer
* Precision is integrating with.
*
********************************************************************************
*/
function CreateCtrlServices()
{
    var domainName = PropertyValue("PRECISION.DOMAIN0");
    var objserv = PropertyValue("OBJECTSERVER0.NAME");

    if (domainName.length > 0 && objserv.length > 0)
    {
        //
        // Set up platform specific variables
        //
        var separator = "/";
        if (ncipackage.shortArch == "win32")
        {
            separator = "\\";
        }

        var replaceRegex = new RegExp("\"NCOMS\"", "g");

        try
        {
            var newFileName = GetNcHome() + separator + "etc" + separator
                            + "precision" + separator + "CtrlServices."
                            + domainName + ".cfg";

            var newFile = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            newFile.initWithPath(newFileName);

            //
            // Only create a CtrlServices.DOMAIN.cfg if there isn't one already.
            //
            if (newFile.exists())
            {
                Log(1, "Not creating " + newFileName + " because it already exists");
            }
            else
            {
                var defaultFileName = GetNcHome() + separator + "etc" + separator
                                    + "precision" + separator + "default" + separator
                                    + "CtrlServices.cfg";

                var defaultFile = Components.classes["@mozilla.org/file/local;1"].
                    createInstance(Components.interfaces.nsILocalFile);
                defaultFile.initWithPath(defaultFileName);

                if (defaultFile.exists())
                {
                    var defaultCtrlServices = ReadFile(defaultFile);

                    var newCtrlServices = defaultCtrlServices.replace(replaceRegex,
                                                                      "\"" + objserv + "\"");

                    var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                        createInstance(Components.interfaces.nsIFileOutputStream);
                    foStream.init(newFile, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
                    foStream.write(newCtrlServices, newCtrlServices.length);
                    foStream.close();
                }
                else
                {
                    Log(1, "Not creating " + newFileName + " because the template "
                         + defaultFileName + " doesn't exist");
                }
            }
        }
        catch(e)
        {
            Log(0, e);
        }
    }
    else
    {
        Log(0, "Unable to create CtrlServices.DOMAIN.cfg as Precision domain "
             + "or object server name are not available.");
    }
}

/**
********************************************************************************
*
* UserIsRoot()
*
* Determine if the user running the installation is root.
*
* @return true if the user running the installation is root; otherwise false.
*
********************************************************************************
*/
function UserIsRoot()
{
    var isRoot = false;

    //
    // Can't use ncipackage.shortArch to determine the architecture
    // here, as it needs to work from both a XUL panel and the package
    // XML.
    //
    // We assume there is only a %windir% environment variable on Windows.
    //
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    var isWin = propMgr.GetProp("$windir").value;
    if (isWin)
    {
        //
        // Windows installs must be done as a user with administrative
        // privileges, so they are the equivalent of root.
        //
        isRoot = true;
    }
    else
    {
        //
        // Run the Unix "id" command to check if this is the root user.
        //
        try
        {
            var scriptFilename = GetNcHome() + "/id.sh";

            var tmpScript = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            tmpScript.initWithPath(scriptFilename);

            var contents = "#!/bin/sh\n"
                + "id > " + GetNcHome() + "/id.tmp\n";

            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(tmpScript, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
            foStream.write(contents, contents.length);
            foStream.close();

            var process = Components.classes["@mozilla.org/process/util;1"].
                createInstance(Components.interfaces.nsIProcess);
            process.init(tmpScript);

            // Run the process.
            // If the first param is true, the calling process will be blocked until
            // the called process terminates.
            // Second and third params are used to pass command-line arguments
            // to the process.
            var args = [];
            process.run(true, args, args.length);

            var outputFilename = GetNcHome() + "/id.tmp";

            var tmpOutput = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            tmpOutput.initWithPath(outputFilename);

            var lines = ReadFile(tmpOutput).split("\n");
            if (lines[0].match(/^uid=0\(/))
            {
                isRoot = true;
            }

            tmpScript.remove(false);
            tmpOutput.remove(false);
        }
        catch(e)
        {
            LogMessage(0, "Cannot determine current user ID");
        }
    }

    return isRoot;
}

