/******************************************************************/
/*                                                                */
/* Copyright (C) 1994 - 2005, Micromuse, Ltd.                     */
/* All Rights Reserved                                            */
/*                                                                */
/*----------------------------------------------------------------*/
/*                                                                */
/* THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE                    */
/* The copyright notice above does not evidence any               */
/* actual or intended publication of such source code.            */
/*                                                                */
/* RESTRICTED RIGHTS:                                             */
/*                                                                */
/* This file may have been supplied under a license.              */
/* It may be used, disclosed, and/or copied only as permitted     */
/* under such license agreement. Any copy must contain the        */
/* above copyright notice and this restricted rights notice.      */
/* Use, copying, and/or disclosure of the file is strictly        */
/* prohibited unless otherwise provided in the license agreement. */
/*                                                                */
/******************************************************************/

/**
********************************************************************************
*
* LogMessage()
*
* Log a message to the installer log file.
*
* @param level The lowest level at which the message should be logged.
*              0 - Errors
*              1 - Warnings
*              2 - Info
*              3 - Debug
*
* @param msg The message to be logged.
*
********************************************************************************
*/
function LogMessage(level, msg)
{
    try
    {
        var logger = Components.classes["@micromuse.com/NCILog;1"].
            getService(Components.interfaces.nsINCILog);

        switch (level)
        {
            case 0:
                logger.error(msg);
                break;
            case 1:
                logger.warning(msg);
                break;
            case 2:
                logger.info(msg);
                break;
            case 3:
                logger.debug(msg);
                break;
            default:
                break;
        }
    }
    catch(e)
    {
        // Logging failure must not affect functionality, so fail silently.
    }
}

/**
********************************************************************************
*
* GetNCHome()
*
* Get the value of the NCHOME environment variable.
*
* @return The value of the NCHOME environment variable.
*
********************************************************************************
*/
function GetNCHome()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    return propMgr.GetProp("$NCHOME").value;
}

/**
********************************************************************************
*
* ReadFile
*
* Read an existing file.  The caller should ensure the file exists before
* calling this function.
*
* @param file The nsIFile object to be read from.
*
* @return The entire file contents as a string.
*
********************************************************************************
*/
function ReadFile(file)
{
    var result = "";

    var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].
        createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].
        createInstance(Components.interfaces.nsIScriptableInputStream);

    try
    {
        fstream.init(file, 1, 0, 0);
        sstream.init(fstream);

        var bytes = fstream.available();
        while (bytes > 0)
        {
            result += sstream.read(bytes);

            bytes = fstream.available();
        }
        sstream.close();
        fstream.close();
    }
    catch(e)
    {
        LogMessage(0, e);
    }

    return result;
}

/**
********************************************************************************
*
* GetExistingObjectServersWindows()
*
* Get an array containing details of all configured object servers on a Windows
* machine.
*
* @return An array of arrays containing details of all configured object servers
*         on the current Windows machine.  The format is:
*         [ [ name1, host1, port1 ], [ name2, host2, port2 ] ]
*         If no object servers are configured, null is returned.
*
********************************************************************************
*/
function GetExistingObjectServersWindows()
{
    var result = null;
    var count = 0;

    //
    // Look for servers configured in Sql.ini
    //
    // Format is:
    //
    // [NAME]
    // master=NLWNSCK,HOST,PORT
    // query=NLWNSCK,HOST,PORT
    //
    var filename = GetNCHome() + "\\ini\\Sql.ini";

    var file = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filename);

    if (file.exists())
    {
        // Parse the file
        var lines = ReadFile(file).split("\n");
        var fields = null;

        var currentServername = "";
        var currentHost = "";
        var currentPort = 0;

        // Strip comments and leading and trailing spaces
        for (var index = 0; index < lines.length; ++index)
        {
            lines[index] = lines[index].replace(/;.*/, "");
            lines[index] = lines[index].replace(/\s*\r*$/, "");
            lines[index] = lines[index].replace(/^\s*/, "");

            // Ignore blank lines
            if (lines[index].length > 0)
            {
                // If it's between square brackets it's a server name
                if (lines[index].charAt(0) == "[" &&
                    lines[index].charAt(lines[index].length - 1) == "]")
                {
                    currentServername = lines[index].substr(1, lines[index].length - 2);
                }
                else
                {
                    // It's not a server name
                    // Tokenise the line
                    fields = lines[index].split(/,/);

                    // There should be 3 fields
                    if (fields.length == 3)
                    {
                        // If the 1st field is master, then
                        // the 2nd and 3rd fields are host and
                        // port respectively
                        if (fields[0] == "master=NLWNSCK")
                        {
                            currentHost = fields[1];
                            currentPort = fields[2];

                            // Store the details and
                            // increment the count
                            if (count == 0)
                            {
                                result = new Array();
                            }
                            result[count] = new Array(currentServername, currentHost, currentPort);
                            ++count;
                        }
                    }
                }
            }
        }
    }

    return result;
}

/**
********************************************************************************
*
* GetUnixPlatformName
*
* Get the platform name in a way that works for both XUL panels and the package
* XML scripts.
*
* We run "uname -s", storing the output in a file.  Then we re-read the file.
*
* @return A string containing the Netcool platform name of the current Unix box,
*         i.e., one of solaris2, linux2x86, aix5 or hpux11.  If an error occurs
*         null is returned.
*
********************************************************************************
*/
function GetUnixPlatformName()
{
    //
    // Workaround for "Red Hat Enterprise Linux AS release 3 (Taroon Update 5)"
    //
    // The main code below will cause the installer to crash on this platform,
    // so instead we assume that if the file /etc/redhat-release exists then
    // this is Linux.
    //
    var rhReleaseFile = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    rhReleaseFile.initWithPath("/etc/redhat-release");

    if (rhReleaseFile.exists())
    {
        return "linux2x86";
    }

    //
    // Main code to run "uname -s"
    //
    var uname = "";

    try
    {
        var scriptFilename = GetNCHome() + "/uname.sh";

        var tmpScript = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        tmpScript.initWithPath(scriptFilename);

        var contents = "#!/bin/sh\n"
            + "uname -s > " + GetNCHome() + "/uname.tmp\n";

        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream.init(tmpScript, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
        foStream.write(contents, contents.length);
        foStream.close();

        var process = Components.classes["@mozilla.org/process/util;1"].
            createInstance(Components.interfaces.nsIProcess);
        process.init(tmpScript);

        // Run the process.
        // If the first param is true, the calling process will be blocked until
        // the called process terminates.
        // Second and third params are used to pass command-line arguments
        // to the process.
        var args = [];
        process.run(true, args, args.length);

        var outputFilename = GetNCHome() + "/uname.tmp";

        var tmpOutput = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        tmpOutput.initWithPath(outputFilename);

        var lines = ReadFile(tmpOutput).split("\n");
        uname = lines[0];

        tmpScript.remove(false);
        tmpOutput.remove(false);
    }
    catch(e)
    {
        LogMessage(0, "Cannot determine current platform name");
    }

    if (uname == "SunOS")
    {
        return "solaris2";
    }
    if (uname == "Linux")
    {
        return "linux2x86";
    }
    if (uname == "HP-UX")
    {
        return "hpux11";
    }
    if (uname == "AIX")
    {
        return "aix5";
    }

    return null;
}

/**
********************************************************************************
*
* GetExistingObjectServersUnix()
*
* Get an array containing details of all configured object servers on a Unix
* machine.
*
* @return An array of arrays containing details of all configured object servers
*         on the current Unix machine.  The format is:
*         [ [ name1, host1, port1 ], [ name2, host2, port2 ] ]
*         If no object servers are configured, null is returned.
*
********************************************************************************
*/
function GetExistingObjectServersUnix()
{
    var result = null;
    var count = 0;

    var platform = GetUnixPlatformName();

    //
    // Look for servers configured in interfaces.ARCH
    //
    // Format is:
    //
    // NAME
    //    master token token HOST PORT
    //    query token token HOST PORT
    //
    var filename = GetNCHome() + "/etc/interfaces." + platform;

    var file = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filename);

    if (file.exists())
    {
        // Parse the file
        var lines = ReadFile(file).split("\n");
        var fields = null;

        var currentServername = "";
        var currentHost = "";
        var currentPort = 0;

        // Strip comments and trailing spaces
        for (var index = 0; index < lines.length; ++index)
        {
            lines[index] = lines[index].replace(/#.*/, "");
            lines[index] = lines[index].replace(/\s*$/, "");

            // Ignore blank lines
            if (lines[index].length > 0)
            {
                // If there is no whitespace at the beginning of the
                // line it's a server name
                if (lines[index].match(/^\s/))
                {
                    // It's not a server name
                    // Tokenise the line
                    fields = lines[index].split(/\s/);

                    // There should be 6 fields
                    if (fields.length == 6)
                    {
                        // If the 2nd field is master, then
                        // the 5th and 6th fields are host and
                        // port respectively
                        if (fields[1] == "master")
                        {
                            currentHost = fields[4];
                            currentPort = fields[5];

                            // Store the details and
                            // increment the count
                            if (count == 0)
                            {
                                result = new Array();
                            }
                            result[count] = new Array(currentServername, currentHost, currentPort);
                            ++count;
                        }
                    }
                }
                else
                {
                    currentServername = lines[index];
                }
            }
        }
    }

    return result;
}

/**
********************************************************************************
*
* GetExistingObjectServers()
*
* Get an array containing details of all configured object servers on the
* current machine.
*
* @return An array of arrays containing details of all configured object servers
*         on the current machine.  The format is:
*         [ [ name1, host1, port1 ], [ name2, host2, port2 ] ]
*         If no object servers are configured, null is returned.
*
********************************************************************************
*/
function GetExistingObjectServers()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);

    //
    // Can't use ncipackage.shortArch to determine the architecture
    // here, as it needs to work from both a XUL panel and the package
    // XML.
    //
    // We assume there is only a %windir% environment variable on Windows.
    //
    var isWin = propMgr.GetProp("$windir").value;
    if (isWin)
    {
        return GetExistingObjectServersWindows();
    }
    else
    {
        return GetExistingObjectServersUnix();
    }
}

/**
********************************************************************************
*
* CreateSqlIni()
*
* Create a Sql.ini file containing details of the currently chosen object
* server.
*
* This function should only be called if Sql.ini doesn't already exist.
*
********************************************************************************
*/
function CreateSqlIni()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);
    var filename = GetNCHome() + "\\ini\\Sql.ini";

    var servername = propMgr.GetProp("OBJECTSERVER0.NAME").value;
    var host = propMgr.GetProp("OBJECTSERVER0.HOST").value;
    var port = propMgr.GetProp("OBJECTSERVER0.PORT").value;

    var contents = "[" + servername + "]\r\n"
        + "master=NLWNSCK," + host + "," + port + "\r\n"
        + "query=NLWNSCK," + host + "," + port + "\r\n";

    try
    {
        var file = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(filename);

        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
        foStream.write(contents, contents.length);
        foStream.close();
    }
    catch(e)
    {
        LogMessage(0, "Unable to write " + filename);
    }
}

/**
********************************************************************************
*
* UpdateOmniDat()
*
* Add the desired object server into omni.dat.
*
* Since this may be the first time omni.dat has been edited, "omnihost" is
* replaced with the name of the local box.
*
********************************************************************************
*/
function UpdateOmniDat()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
        getService(Components.interfaces.nsINCIProperties);
    var filename = GetNCHome() + "/etc/omni.dat";

    var servername = propMgr.GetProp("OBJECTSERVER0.NAME").value;
    var host = propMgr.GetProp("OBJECTSERVER0.HOST").value;
    var port = propMgr.GetProp("OBJECTSERVER0.PORT").value;

    var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(filename);

    var contents = "";
    var needToAdd = true;

    if (file.exists())
    {
        // Find the name of the current box
        var DNSService = Components.classes["@mozilla.org/network/dns-service;1"].
            getService(Components.interfaces.nsIDNSService);

        var thisHostname = DNSService.myHostName;

        // Parse the file
        var lines = ReadFile(file).split("\n");
        var fields = null;

        var currentServername = "";
        var currentHost = "";
        var currentPort = 0;

        var comment = "";
        var workSpace = "";
        var skipping = false;

        // Strip comments and trailing spaces
        for (var index = 0; index < lines.length; ++index)
        {
            workSpace = lines[index].replace(/\s*#.*/, "");
            if (workSpace.length < lines[index].length)
            {
                comment = lines[index].substr(workSpace.length);
            }
            else
            {
                comment = "";
            }

            if (skipping)
            {
                if (workSpace.indexOf("}") > -1)
                {
                    skipping = false;
                }

                if (comment.length > 0)
                {
                    contents += comment + "\n";
                }
            }
            else
            {
                if (workSpace == "[" + servername + "]")
                {
                    workSpace += "\n"
                        + "{\n"
                        + "\tPrimary: " + host + " " + port + "\n"
                        + "}";

                    skipping = true;
                    needToAdd = false;
                }
                else
                {
                    workSpace = workSpace.replace(/omnihost/, thisHostname);
                }

                contents += workSpace + comment + "\n";
            }
        }
    }

    if (needToAdd)
    {
        contents += "[" + servername + "]\n"
            + "{\n"
            + "\tPrimary: " + host + " " + port + "\n"
            + "}\n";
    }

    try
    {
        var file = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(filename);

        var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
            createInstance(Components.interfaces.nsIFileOutputStream);
        foStream.init(file, 0x02 | 0x08 | 0x20, 0666, 0); // write, create, truncate
        foStream.write(contents, contents.length);
        foStream.close();
    }
    catch(e)
    {
        LogMessage(0, "Unable to write " + filename);
    }

    return null;
}

/**
********************************************************************************
*
* RunNcoIGen()
*
* Convert omni.dat into interfaces.ARCH by running nco_igen -arch ARCH.
*
* @return true if nco_igen is run, and false if it isn't.
*
********************************************************************************
*/
function RunNcoIGen()
{
    var ncoIGen = GetNCHome() + "/bin/nco_igen";

    var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath(ncoIGen);

    if (file.exists())
    {
        // Need to create a script wrapper to ensure that
        // OMNIHOME is set correctly when nco_igen is run.
        var scriptFilename = GetNCHome() + "/nco_igen.sh";

        var tmpScript = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        tmpScript.initWithPath(scriptFilename);

        var contents = "#!/bin/sh\n"
            + "OMNIHOME=" + GetNCHome() + "\n"
            + "export OMNIHOME\n"
            + ncoIGen + " -arch " + ncipackage.shortArch + "\n";

        try
        {
            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(tmpScript, 0x02 | 0x08 | 0x20, 0777, 0); // write, create, truncate
            foStream.write(contents, contents.length);
            foStream.close();

            var process = Components.classes["@mozilla.org/process/util;1"].
                createInstance(Components.interfaces.nsIProcess);
            process.init(tmpScript);

            // Run the process.
            // If the first param is true, the calling process will be blocked until
            // the called process terminates.
            // Second and third params are used to pass command-line arguments
            // to the process.
            var args = [];
            process.run(true, args, args.length);

            tmpScript.remove(false);
        }
        catch(e)
        {
            LogMessage(0, e);
        }

        return true;
    }
    else
    {
        LogMessage(0, "Could not find " + ncoIGen + " and hence could not create the interfaces file.");

        return false;
    }
}

/**
********************************************************************************
*
* CreateInterfacesArch()
*
* Create an interfaces.ARCH file by updating omni.dat and then running nco_igen.
*
********************************************************************************
*/
function CreateInterfacesArch()
{
    UpdateOmniDat();

    RunNcoIGen();
}

/**
********************************************************************************
*
* CreateConfigFile()
*
* Create an interfaces.ARCH or Sql.ini file (depending on the current platform)
* containing the details of how to contact the object server.
*
********************************************************************************
*/
function CreateConfigFile()
{
    if (ncipackage.shortArch == "win32")
    {
        CreateSqlIni();
    }
    else
    {
        CreateInterfacesArch();
    }
}

